from pydantic import BaseModel
from datetime import datetime
from typing import List, Optional

class Address(BaseModel):
    id: Optional[str]
    customer_id: Optional[str]
    first_name: Optional[str]
    last_name: Optional[str]
    company: Optional[str]
    address1: Optional[str]
    address2: Optional[str]
    city: Optional[str]
    province: Optional[str]
    country: Optional[str]
    zip: Optional[str]
    phone: Optional[str]
    name: Optional[str]
    province_code: Optional[str]
    country_code: Optional[str]
    country_name: Optional[str]
    default: Optional[str]


class CustomerShopify(BaseModel):
    id: Optional[int]
    email: Optional[str]
    accepts_marketing: Optional[str]
    created_at: datetime
    updated_at: datetime
    first_name: Optional[str]
    last_name: Optional[str]
    orders_count: Optional[int]
    state: Optional[str]
    total_spent: Optional[float]
    last_order_id: Optional[str]
    note: Optional[str]
    verified_email: Optional[bool]
    multipass_identifier: Optional[str]
    tax_exempt: Optional[str]
    phone: Optional[str]
    tags: Optional[str]
    last_order_name: Optional[str]
    currency: Optional[str]
    addresses: Optional[List[Optional[Address]]]
    accepts_marketing_updated_at: Optional[datetime]
    marketing_opt_in_level: Optional[str]
    tax_exemptions: Optional[List[Optional[str]]]
    admin_graphql_api_id: Optional[str]



from shopipy.exceptions import *
from shopipy.resources import webhooks


class ShopiPy:
    """ SDK Class to interact with the Shopify API. """

    def __init__(
        self,
        shop_domain: str = None,
        version: str = "2021-04",
        api_key: str = None,
        api_secret: str = None,
        app_key: str = None,
        app_password: str = None,
        app_shared_secret: str = None,
        access_token: str = None
    ):
        """
        Initial SDK Setup

        Args:
            shop_domain (str): [description]
            version (str, optional): [description]. Defaults to "2021-04".
            api_key (str, optional): The App API Key. Defaults to None.
            api_secret (str, optional): The App API Secret. Defaults to None.
            app_key (str, optional): The Private App App Key. Defaults to None.
            app_password (str, optional): The Private App Password. Defaults to None.
            app_shared_secret (str, optional): The Private App Shared Secret, used to validate HMAC. Defaults to None.
            access_token (str, optional): [description]. Defaults to None.
        """
        self.config = dict(
            shop_domain=shop_domain,
            access_token=access_token,
            api_key=api_key,
            api_secret=api_secret,
            app_key=app_key,
            app_password=app_password,
            app_shared_secret=app_shared_secret,
            version=version
        )

    @property
    def OAuth(self):
        from shopipy.resources.oauth import OAuth
        return OAuth(**self.config)

    @property
    def Orders(self):
        from shopipy.resources.orders import Orders
        return Orders(**self.config)

    @property
    def Products(self):
        from shopipy.resources.products import Products
        return Products(**self.config)

    @property
    def Customers(self):
        from shopipy.resources.customers import Customers
        return Customers(**self.config)

    @property
    def Collections(self):
        from shopipy.resources.collections import Collections
        return Collections(**self.config)

    @property
    def Inventory(self):
        from shopipy.resources.inventory import Inventory
        return Inventory(**self.config)

    @property
    def Fulfillment(self):
        from shopipy.resources.fulfillment import Fulfillment
        return Fulfillment(**self.config)

    @property
    def Shipping(self):
        from shopipy.resources.shipping import Shipping
        return Shipping(**self.config)

    @property
    def Webhooks(self):
        from shopipy.resources.webhooks import Webhooks
        return Webhooks(**self.config)



from datetime import datetime
from shopipy.base import BaseClient
from shopipy.resources.products.schemas import Product # type: ignore

class Products(BaseClient):
    product_path = "/products"

    def __init__(self, *args, **kwargs):
        super(Products, self).__init__(*args, **kwargs)

    def list_products(
        self,
        collection_id: str = None,
        created_at_max: datetime = None,
        created_at_min: datetime = None,
        fields: str = None,
        handle: str = None,
        ids: str = None,
        limit: int = 250,
        presentment_currencies: datetime = None,
        product_type: datetime = None,
        published_at_max: datetime = None,
        published_at_min: datetime = None,
        published_status: str = None,
        since_id: int = None,
        status: str = None,
        title: str = None,
        vendor: str = None,
        updated_at_max: datetime = None,
        updated_at_min: datetime = None,
    ):
        params = dict(
            collection_id=collection_id,
            created_at_max=created_at_max,
            created_at_min=created_at_min,
            fields=fields,
            handle=handle,
            ids=ids,
            limit=limit,
            presentment_currencies=presentment_currencies,
            product_type=product_type,
            published_at_max=published_at_max,
            published_at_min=published_at_min,
            published_status=published_status,
            since_id=since_id,
            status=status,
            title=title,
            vendor=vendor,
            updated_at_max=updated_at_max,
            updated_at_min=updated_at_min
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.product_path}.json", **cleaned)

    def get(self, product_id: str, fields: str = None):
        params = dict(product_id=product_id, fields=fields)
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.product_path}/{product_id}.json", **cleaned)

    def count(
        self,
        collection_id: str = None,
        created_at_max: datetime = None,
        created_at_min: datetime = None,
        product_type: datetime = None,
        published_at_max: datetime = None,
        published_at_min: datetime = None,
        published_status: str = None,
        vendor: str = None,
        updated_at_max: datetime = None,
        updated_at_min: datetime = None,
    ):
        params = dict(
            collection_id=collection_id,
            created_at_max=created_at_max,
            created_at_min=created_at_min,
            product_type=product_type,
            published_at_max=published_at_max,
            published_at_min=published_at_min,
            published_status=published_status,
            vendor=vendor,
            updated_at_max=updated_at_max,
            updated_at_min=updated_at_min
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.product_path}/count.json", **cleaned)

    def save(self, product: Product):
        payload = product.dict()
        return self._post(path=f"{self.product_path}.json", payload=payload)

    def update(self, product: Product):
        payload = product.dict()
        return self._put(path=f"{self.product_path}.json", payload=payload)

    def delete(self, product_id: str):
        return self._delete(path=f"{self.product_path}/{product_id}.json")



from typing import List
from shopipy.base import BaseClient
from .schemas import InventoryItem

class Inventory(BaseClient):
    inv_items = "/inventory_items"
    inv_levels = "/inventory_levels"
    inv_locations = "/locations"

    def __init__(self, *args, **kwargs):
        super(Inventory, self).__init__(*args, **kwargs)

    # Inventory Item
    def list_items(self, ids: List[str] = None, limit: int = 250):
        if not ids:
            raise Exception("IDs not found. You need to provide the items' id's to get the inventory level.")
        list_path= self.inv_items + ".json"
        parsed_ids = ",".join(ids)
        return self._get(path=list_path, ids=parsed_ids, limit=limit)

    def get_item(self, item_id: str):
        item_path = self.inv_items + f"/{item_id}.json"
        return self._get(path=item_path)

    def update_item(self, item_id: str, item_payload: InventoryItem):
        item_path = self.inv_items + f"/{item_id}.json"
        return self._put(path=item_path, payload=item_payload.dict())

    # Inventory Item Level
    def get_items_level(self):
        item_level = self.inv_levels + f".json"
        return self._get(path=item_level)

    def set_level(self, available: int, inventory_item_id: int, location_id: int, disconnect_if_necessary: bool = False):
        payload = dict(
            available=available,
            inventory_item_id=inventory_item_id,
            location_id=location_id,
            disconnect_if_necessary=disconnect_if_necessary
        )
        full_path = self.inv_levels + "/set.json"
        return self._post(path=full_path, payload=payload)

    def adjust_level(self, available_adjustment: int, inventory_item_id: int, location_id: int):
        payload = dict(
            available_adjustment=available_adjustment,
            inventory_item_id=inventory_item_id,
            location_id=location_id
        )
        full_path = self.inv_levels + "/adjust.json"
        return self._post(path=full_path, payload=payload)

    def delete_level(self, inventory_item_id: int, location_id: int):
        full_path = self.inv_levels + ".json"
        return self._delete(path=full_path, inventory_item_id=inventory_item_id, location_id=location_id)

    def connect_item(self, location_id: int, inventory_item_id: int, relocate_if_necessary: bool = False):
        payload = dict(
            location_id=location_id,
            inventory_item_id=inventory_item_id,
            relocate_if_necessary=relocate_if_necessary
        )
        full_path = self.inv_levels + "/connect.json"
        return self._post(path=full_path, payload=payload)

    # Inventory Locations
    def list_locations(self):
        location_path = self.inv_locations + f".json"
        return self._get(path=location_path)

    def get_location(self, location_id: str):
        location_path = self.inv_locations + f"/{location_id}.json"
        return self._get(path=location_path)

    def get_location_levels(self, location_id: str):
        location_path = self.inv_locations + f"/{location_id}/inventory_levels.json"
        return self._get(path=location_path)

    def count_locations(self):
        location_path = self.inv_locations + f"/count.json"
        return self._get(path=location_path)


from datetime import datetime
from shopipy.base import BaseClient

class Collections(BaseClient):
    collection_path = "/collection"

    def __init__(self, *args, **kwargs):
        super(Collections, self).__init__(*args, **kwargs)

    def list_collections(self):
        count_path = self.collection_path + ".json"
        return self._get(path=count_path)

    def get_collection(self, item_id: str = "any"):
        item_path = self.collection_path + f"/{item_id}.json"
        return self._get(path=item_path)

    def get_collection_orders(self):
        item_level = self.collection_path + f".json"
        return self._get(path=item_level)

    def count_collections(self):
        raise NotImplementedError("Collections Adjustment Not Implemented.")

    def search_collection(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")

    # Addresses
    def list_collection_addresses(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")

    def get_collection_address(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")

    def set_collection_address(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")

    def set_collection_default_address(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")

    def delete_collection_address(self):
        raise NotImplementedError("Set Collections Level Not Implemented.")




from datetime import datetime
from typing import Any, Dict, Optional
from pydantic import BaseModel

class Transactions(BaseModel):

    id: Optional[int]
    order_id: Optional[int]
    kind: Optional[str]
    gateway: Optional[str]
    status: Optional[str]
    message: Optional[str]
    created_at: Optional[datetime]
    test: bool
    authorization: Optional[str]
    location_id: Optional[int]
    user_id: Optional[int]
    parent_id: Optional[int]
    processed_at: Optional[datetime]
    device_id: Optional[int]
    receipt: Optional[Dict[Any, Any]]
    error_code: Optional[str]
    source_name: Optional[str]
    forex_adjustment: Optional[str]
    amount: Optional[str]
    currency: Optional[str]
    admin_graphql_api_id: Optional[str]



from datetime import datetime
from shopipy.base import BaseClient

class Collections(BaseClient):
    collection_path = "/collection"

    def __init__(self, *args, **kwargs):
        super(Collections, self).__init__(*args, **kwargs)

    def list_collections(self):
        raise NotImplementedError

    def get_collection(self, item_id: str = "any"):
        raise NotImplementedError

    def get_collection_products(self):
        raise NotImplementedError

    def count_collections(self):
        raise NotImplementedError

    def search_collection(self):
        raise NotImplementedError


from shopipy.base import BaseClient
import requests
from typing import Any, Dict, List
import hmac, urllib, json, six, time
from hashlib import sha256

class OAuth(BaseClient):

    def __init__(self, shop_domain: str = None, version: str = "2021-04", api_key: str = None, api_secret: str = None, app_key: str = None, app_password: str = None, app_shared_secret: str = None, access_token: str = None, timeout: int = 5):
        super().__init__(shop_domain=shop_domain, version=version, api_key=api_key, api_secret=api_secret, app_key=app_key, app_password=app_password, app_shared_secret=app_shared_secret, access_token=access_token, timeout=timeout)

    def create_permission_url(self, shop_domain: str, scopes: List[str], redirect_uri: str, state: str = None):
        """ Creates the URL that will be used for the OAuth flow """
        query_params = dict(
            client_id=self.api_key,
            scope=",".join(scopes),
            redirect_uri=redirect_uri,
            state=state
        )
        clened_params = self._clean_params(query_params)
        return f"https://{shop_domain}.myshopify.com/admin/oauth/authorize?{urllib.parse.urlencode(clened_params)}"

    def request_token(self, request_params: Dict[Any, Any]):
        """ Parses the callback from Shopify to get the token after the OAuth Flow is complete """
        if self.access_token:
            return self.access_token
        
        if not self.validate_params(request_params):
            raise Exception("Invalid HMAC: Possibly malicious login")

        url = f"https://{self.shop_domain}.myshopify.com/admin/oauth/access_token?"
        
        query_params = dict(
            client_id=self.api_key,
            client_secret=self.api_secret,
            code=request_params["code"]
        )
        
        response = requests.get(url, params=query_params)
        if response.status_code == 200:
            json_payload = response.json()
            self.token = json_payload["access_token"]
            self.scopes = json_payload["scope"]
            return self.token
        else:
            raise Exception(response.text)

    def validate_params(self, params: Dict[Any, Any]):
        """ Avoid replay attacks by making sure the request isn't more than a day old. """
        one_day = 24 * 60 * 60
        if int(params.get("timestamp", 0)) < time.time() - one_day:
            return False

        return self.validate_hmac(params)

    def validate_hmac(self, params: Dict[Any, Any]):
        """ Validates that the request is from Shopify through HMAC validation """
        if "hmac" not in params:
            return False

        hmac_calculated = self.calculate_hmac(params).encode("utf-8")
        hmac_to_verify = params["hmac"].encode("utf-8")

        # Try to use compare_digest() to reduce vulnerability to timing attacks.
        # If it's not available, just fall back to regular string comparison.
        try:
            return hmac.compare_digest(hmac_calculated, hmac_to_verify)
        except AttributeError:
            return hmac_calculated == hmac_to_verify

    def calculate_hmac(self, params: Dict[Any, Any]):
        """
        Calculate the HMAC of the given parameters in line with Shopify's rules for OAuth authentication.
        See http://docs.shopify.com/api/authentication/oauth#verification.
        """
        encoded_params = self.__encoded_params_for_signature(params)
        return hmac.new(self.api_secret.encode(), encoded_params.encode(), sha256).hexdigest()

    def __encoded_params_for_signature(self, params: Dict[Any, Any]):
        """
        Sort and combine query parameters into a single string, excluding those that should be removed and joining with '&'
        """

        def encoded_pairs(params):
            for k, v in six.iteritems(params):
                if k == "hmac":
                    continue

                if k.endswith("[]"):
                    # foo[]=1&foo[]=2 has to be transformed as foo=["1", "2"] note the whitespace after comma
                    k = k.rstrip("[]")
                    v = json.dumps(list(map(str, v)))

                # escape delimiters to avoid tampering
                k = str(k).replace("%", "%25").replace("=", "%3D")
                v = str(v).replace("%", "%25")
                yield "{0}={1}".format(k, v).replace("&", "%26")

        return "&".join(sorted(encoded_pairs(params)))



from pydantic.main import BaseModel
from typing import List, Optional
from datetime import datetime


class Option(BaseModel):
    option1: Optional[str]
    option2: Optional[str]
    option3: Optional[str]


class Price(BaseModel):
    currency_code: Optional[str]
    amount: Optional[str]


class CompareAtPrice(Price):
    pass


class PresentmentPrices(BaseModel):
    price: Optional[Price]
    compare_at_price: Optional[CompareAtPrice]


class Options(BaseModel):
    id: Optional[str]
    product_id: Optional[str]
    name: Optional[str]
    position: Optional[str]
    values: Optional[str]


class Image(BaseModel):
    id: Optional[int]
    product_id: Optional[int]
    position: Optional[int]
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    alt: Optional[str]
    width: Optional[int]
    height: Optional[int]
    src: Optional[str]
    variant_ids: Optional[int]
    admin_graphql_api_id: Optional[str]


class Variant(BaseModel):
    id: Optional[int]
    product_id: Optional[int]
    title: Optional[str]
    price: Optional[str]
    sku: Optional[str]
    position: Optional[int]
    inventory_policy: Optional[str]
    compare_at_price: Optional[str]
    fulfillment_service: Optional[str]
    inventory_management: Optional[str]
    option: Optional[Option]
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    taxable: Optional[bool]
    barcode: Optional[str]
    grams: Optional[int]
    image_id: Optional[int]
    weight: Optional[int]
    weight_unit: Optional[str]
    inventory_item_id: Optional[int]
    inventory_quantity: Optional[int]
    inventory_quantity_adjustment: Optional[int]
    old_inventory_quantity: Optional[int]
    requires_shipping: Optional[bool]
    admin_graphql_api_id: Optional[str]
    presentment_prices: Optional[List[PresentmentPrices]]
    tax_code: Optional[str]


class Product(BaseModel):
    id: Optional[int]
    title: Optional[str]
    body_html: Optional[str]
    vendor: Optional[str]
    product_type: Optional[str]
    collection_id: Optional[str]
    created_at: Optional[datetime]
    handle: Optional[str]
    updated_at: Optional[datetime]
    published_at: Optional[datetime]
    template_suffix: Optional[str]
    tags: Optional[str]
    published_scope: Optional[str]
    admin_graphql_api_id: Optional[str]
    variants: Optional[List[Variant]]
    options: Optional[List[Optional[Option]]]
    images: Optional[List[Optional[Image]]]
    status: Optional[str]




from datetime import datetime
from shopipy.base import BaseClient

class Fulfillment(BaseClient):
    fulfillment_path = "/fulfillment"

    def __init__(self, *args, **kwargs):
        super(Fulfillment, self).__init__(*args, **kwargs)

    def list_fulfillments(self):
        count_path = self.fulfillment_path + ".json"
        return self._get(path=count_path)

    def get_fulfillment(self, item_id: str = "any"):
        item_path = self.fulfillment_path + f"/{item_id}.json"
        return self._get(path=item_path)

    def get_fulfillment_orders(self):
        item_level = self.fulfillment_path + f".json"
        return self._get(path=item_level)

    def count_fulfillments(self):
        raise NotImplementedError("Fulfillment Adjustment Not Implemented.")

    def search_fulfillment(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")

    # Addresses
    def list_fulfillment_addresses(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")

    def get_fulfillment_address(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")

    def set_fulfillment_address(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")

    def set_fulfillment_default_address(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")

    def delete_fulfillment_address(self):
        raise NotImplementedError("Set Fulfillment Level Not Implemented.")




from typing import Optional
from pydantic import BaseModel


class ShopMoney(BaseModel):
    amount: Optional[str]
    currency_code: Optional[str]


class AmountSet(BaseModel):
    shop_money: Optional[ShopMoney]
    presentment_money: Optional[ShopMoney]


class TaxItem(BaseModel):
    title: Optional[str]
    price: Optional[str]
    price_set: Optional[AmountSet]
    rate: Optional[str]
    channel_liable: Optional[str]


from typing import Any, Dict
import requests # type: ignore
from shopipy.exceptions import *

class BaseClient:

    def __init__(
        self,
        shop_domain: str = None,
        version: str = "2021-04",
        api_key: str = None,
        api_secret: str = None,
        app_key: str = None,
        app_password: str = None,
        app_shared_secret: str = None,
        access_token: str = None,
        timeout: int = 5
    ):
        """
        Base class to interact with Shopify API.

        Args:
            shop_domain (str): The shopDomain (without 'myshopify')
            version (str): API Version, defaults to "2021-04"
            access_token (str, optional): access_token, used for public apps.
            api_key (str, optional): API Key, for private apps.
            api_password (str, optional): API Password for private apps.
        """
        super(BaseClient, self).__init__()
        self.shop_domain = shop_domain
        self.version = version
        self.api_key = api_key
        self.api_secret = api_secret
        self.app_key = app_key
        self.app_password = app_password
        self.app_shared_secret = app_shared_secret
        self.access_token = access_token
        self.timeout = timeout
        
        # Calculated fields
        self.base_url = f"https://{shop_domain}.myshopify.com/admin/api/{version}"
        self.header = dict() # type: ignore
        self.private = False
        self.setup()

    def setup(self):
        """
        Main setup function, used to set the API version.
        If the application is private, uses the app_password as the access_token.
        """
        if self.app_password:
            self.private = True
            self.access_token = self.app_password

        # Has to have one or the other (public or private app)        
        if not (self.app_key or self.api_key):
            raise CredentialsError("Can't use API Key when defining the token.")
            
        # App is private but no password was set
        if self.app_key and not self.app_password:
            raise CredentialsError("No password set for private app.")

        # Sets the header for authorized requests
        self.header = {
            "Content-Type": "application/json",
            "X-Shopify-Access-Token": self.access_token
        }


    def _clean_params(self, data: dict):
        """Receives a dictionary of data, and excludes unset.

        Args:
            data (dict): The data dictionary

        Returns:
            data (dict): The cleaned up dictionary
        """
        return dict(filter(lambda item: item[1] is not None, data.items()))

    def _get(self, path: str, **params):
        """
        GET Request to the provided endpoint

        Args:
            path (str): The Shopify endpoint.

        Returns:
            Response: Request's JSON Response object.
        """
        full_path = self.base_url + path
        r = requests.get(url=full_path, headers=self.header, timeout=self.timeout, params=dict(**params))
        response = r.json()
        return response

    def _post(self, path: str, payload: Dict[Any, Any]):
        """
        POST Request to the provided endpoint

        Args:
            path (str): The Shopify endpoint.
            payload (dict): A Payload to send Shopify.

        Returns:
            Response: Request's JSON Response object.
        """    
        full_path = self.base_url + path
        response = requests.post(url=full_path, headers=self.header, json=payload, timeout=self.timeout)
        return response

    def _put(self, path: str, payload: Dict[Any, Any]):
        """
        PUT Request to the provided endpoint

        Args:
            path (str): The Shopify endpoint.
            payload (dict): A Payload to send Shopify.

        Returns:
            Response: Request's JSON Response object.
        """    
        full_path = self.base_url + path
        response = requests.put(url=full_path, headers=self.header, json=payload, timeout=self.timeout)
        return response

    def _delete(self, path: str, **params):
        """
        DELETE Request to the provided endpoint

        Args:
            path (str): The Shopify endpoint.
            payload (dict): A Payload to send Shopify.

        Returns:
            Response: Request's JSON Response object.
        """    
        full_path = self.base_url + path
        response = requests.delete(url=full_path, headers=self.header, timeout=self.timeout, params=dict(**params))
        return response



from pydantic import BaseModel
from datetime import datetime
from typing import List, Optional

class Fulfillments(BaseModel):

    created_at: Optional[str]
    id: Optional[str]
    order_id: Optional[str]
    status: Optional[str]
    tracking_company: Optional[str]
    tracking_number: Optional[str]
    updated_at: Optional[str]



from typing import Optional
from pydantic import BaseModel


class Image(BaseModel):
    created_at: Optional[str]
    alt: Optional[str]
    width: Optional[str]
    height: Optional[str]
    src: Optional[str]


class Collection(BaseModel):
    id: Optional[str]
    handle: Optional[str]
    title: Optional[str]
    updated_at: Optional[str]
    body_html: Optional[str]
    published_at: Optional[str]
    sort_order: Optional[str]
    template_suffix: Optional[str]
    products_count: Optional[str]
    collection_type: Optional[str]
    published_scope: Optional[str]
    admin_graphql_api_id: Optional[str]
    image: Optional[Image]



from datetime import datetime
from shopipy.base import BaseClient

class Customers(BaseClient):
    customer_path = "/customers"

    def __init__(self, *args, **kwargs):
        super(Customers, self).__init__(*args, **kwargs)

    def get_all(
        self,
        created_at_max: str = None,
        created_at_min: str = None,
        fields: str = None,
        ids: str = None,
        limit: int = 250,
        since_id: int = 0,
        updated_at_max: str = None,
        updated_at_min: str = None
    ):
        params = dict(
            created_at_max=created_at_max,
            created_at_min=created_at_min,
            fields=fields,
            ids=ids,
            limit=limit,
            since_id=since_id,
            updated_at_max=updated_at_max,
            updated_at_min=updated_at_min
        )
        count_path = self.customer_path + ".json"
        return self._get(path=count_path, params=dict(**params))

    def get_customer(self, customer_id: str, fields: str = None):
        full_path = self.customer_path + f"/{customer_id}.json"
        return self._get(path=full_path, params=dict(fields=fields))

    def get_customer_orders(self):
        item_level = self.customer_path + f".json"
        return self._get(path=item_level)

    def count(self):
        raise NotImplementedError("Customer Adjustment Not Implemented.")

    def search(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")

    # Addresses
    def list_customer_addresses(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")

    def get_customer_address(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")

    def set_customer_address(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")

    def set_customer_default_address(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")

    def delete_customer_address(self):
        raise NotImplementedError("Set Customer Level Not Implemented.")




from enum import Enum
from datetime import datetime
from typing import List
from shopipy.base import BaseClient
from .topics import ALL_TOPICS

class Webhooks(BaseClient):
    webhooks_path = "/webhooks"

    def __init__(self, *args, **kwargs):
        super(Webhooks, self).__init__(*args, **kwargs)

    def get_webhooks(
        self,
        address: str = None,
        created_at_max: datetime = None,
        created_at_min: datetime = None,
        fields: str = None,
        limit: int = 250,
        since_id: int = None,
        topic: str = None,
        updated_at_max: datetime = None,
        updated_at_min: datetime = None
    ):
        params = dict(
            address=address,
            created_at_max=created_at_max,
            created_at_min=created_at_min,
            fields=fields,
            topic=topic,
            limit=limit,
            since_id=since_id,
            updated_at_max=updated_at_max,
            updated_at_min=updated_at_min
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.webhooks_path}.json", **cleaned)

    def get_webhook(
        self,
        webhook_id: str = None,
        fields: str = None
    ):
        params = dict(
            webhook_id=webhook_id,
            fields=fields
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.webhooks_path}.json", **cleaned)

    def count(
        self,
        address: str = None,
        topic: str = None
    ):
        params = dict(
            address=address,
            topic=topic
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.webhooks_path}/count.json", **cleaned)

    def set_webhook(
        self,
        topic: str,
        address: str,
        format: str = "json",
        version: str = "2021-04",
        metafield_namespaces: str = None,
        private_metafield_namespaces: str = None
    ):
        """ Sets a single webhook. """
        if topic not in ALL_TOPICS:
            raise TypeError("Invalid topic.")
        params = dict(
            topic=topic,
            address=address,
            format=format,
            version=version,
            metafield_namespaces=metafield_namespaces,
            private_metafield_namespaces=private_metafield_namespaces
        )
        cleaned_params = self._clean_params(params)
        payload = dict(webhook=cleaned_params)
        full_path = f"{self.webhooks_path}.json"
        return self._post(path=full_path, payload=payload)

    def set_webhooks(
        self,
        topics: List[str],
        address: str,
        format: str = "json",
        metafield_namespaces: str = None,
        private_metafield_namespaces: str = None
    ):
        """ Sets a list of webhook topics for the same target address. """
        for each in topics:
            params = dict(
                topic=each,
                address=address,
                format=format,
                metafield_namespaces=metafield_namespaces,
                private_metafield_namespaces=private_metafield_namespaces
            )
            full_path = f"/{self.webhooks_path}.json"
            self._post(path=full_path, payload=params)
        return

    def update_webhook(self):
        raise NotImplementedError

    def remove_webhook(self, webhook_id: str):
        """ Deletes a Webhook. """
        full_path = f"{self.webhooks_path}/{webhook_id}.json"
        return self._delete(path=full_path)

    def remove_all_webhooks(self):
        """ Deletes a Webhook. """
        # Get list of webhooks IDs
        # Make DELETE request to remove the specified ID
        pass



from pydantic import BaseModel
from datetime import datetime
from typing import List, Optional
from shopipy.resources.common.schemas import AmountSet, TaxItem

class Shipping(BaseModel):
    code: Optional[str]
    discounted_price: Optional[str]
    discounted_price_set: Optional[str]
    price: Optional[str]
    price_set: Optional[AmountSet]
    source: Optional[str]
    title: Optional[str]
    tax_lines: List[Optional[TaxItem]]
    carrier_identifier: Optional[str]
    requested_fulfillment_service_id: Optional[str]



from typing import List, Optional
from datetime import datetime
from pydantic import BaseModel
from shopipy.resources.fulfillment.schemas import Fulfillments # type: ignore
from shopipy.resources.transactions.schemas import Transactions # type: ignore
from shopipy.resources.customers.schemas import Address, CustomerShopify # type: ignore
from shopipy.resources.common.schemas import ShopMoney, AmountSet


class OrderAdjustments(BaseModel):
    id: Optional[str]
    order_id: Optional[str]
    refund_id: Optional[str]
    amount: Optional[str]
    tax_amount: Optional[str]
    kind: Optional[str]
    reason: Optional[str]
    amount_set: AmountSet
    tax_amount_set: AmountSet


class DiscountApplications(BaseModel):
    type: Optional[str]
    value: Optional[str]
    value_type: Optional[str]
    allocation_method: Optional[str]
    target_selection: Optional[str]
    target_type: Optional[str]
    description: Optional[str]
    title: Optional[str]


class DiscountCodes(BaseModel):
    code: Optional[str]
    amount: Optional[str]
    type: Optional[str]


class Properties(BaseModel):
    key: Optional[str]
    value: Optional[str]


class TaxLines(BaseModel):
    price: Optional[str]
    rate: Optional[str]
    title: Optional[str]


class DiscountAllocations(BaseModel):

    amount: Optional[str]
    disc_app_index: Optional[str]
    amount_set: Optional[AmountSet]


class LineItems(BaseModel):

    id: Optional[str]
    variant_id: Optional[str]
    title: Optional[str]
    quantity: Optional[str]
    sku: Optional[str]
    variant_title: Optional[str]
    vendor: Optional[str]
    fulfillment_service: Optional[str]
    product_id: Optional[str]
    requires_shipping: Optional[str]
    taxable: Optional[str]
    gift_card: Optional[str]
    name: Optional[str]
    variant_inventory_management: Optional[str]
    properties: Optional[List[Properties]]
    product_exists: Optional[str]
    fulfillable_quantity: Optional[str]
    grams: Optional[str]
    price: Optional[str]
    total_discount: Optional[str]
    fulfillment_status: Optional[str]
    price_set: Optional[AmountSet]
    total_discount_set: Optional[AmountSet]
    discount_allocations: Optional[List[DiscountAllocations]]
    tax_lines: Optional[List[TaxLines]]



class ShippingLine(BaseModel):
    id: Optional[str]
    title: Optional[str]
    price: Optional[str]
    code: Optional[str]
    source: Optional[str]
    phone: Optional[str]
    requested_fulfillment_service_id: Optional[str]
    delivery_category: Optional[str]
    carrier_identifier: Optional[str]
    discounted_price: Optional[str]
    price_set: Optional[AmountSet]
    discounted_price_set: Optional[AmountSet]
    discount_allocations: Optional[List[DiscountAllocations]]
    tax_lines: Optional[TaxLines]


class RefundItem(BaseModel):
    id: Optional[int]
    line_item: Optional[LineItems]
    line_item_id: Optional[int]
    quantity: Optional[int]
    restock_type: Optional[str]
    location_id: Optional[int]
    subtotal: Optional[float]
    total_tax: Optional[float]
    subtotal_set: Optional[AmountSet]
    total_tax_set: Optional[AmountSet]


class Refunds(BaseModel):
    id: Optional[str]
    order_id: Optional[str]
    created_at: Optional[datetime]
    note: Optional[str]
    user_id: Optional[str]
    processed_at: Optional[str]
    refund_line_items: Optional[List[Optional[RefundItem]]]
    transactions: Optional[List[Optional[Transactions]]]
    order_adjustments: Optional[Optional[List[OrderAdjustments]]]


class ShopifyOrders(BaseModel):

    id: Optional[int]
    email: Optional[str]
    closed_at: Optional[str]
    created_at: Optional[datetime]
    updated_at: Optional[str]
    number: Optional[str]
    note: Optional[str]
    token: Optional[str]
    gateway: Optional[str]
    test: Optional[str]
    total_price: Optional[str]
    subtotal_price: Optional[str]
    total_weight: Optional[str]
    total_tax: Optional[str]
    taxes_included: Optional[str]
    currency: Optional[str]
    financial_status: Optional[str]
    confirmed: Optional[str]
    total_discounts: Optional[str]
    total_line_items_price: Optional[str]
    cart_token: Optional[str]
    buyer_accepts_marketing: Optional[str]
    name: Optional[str]
    referring_site: Optional[str]
    landing_site: Optional[str]
    cancelled_at: Optional[str]
    cancel_reason: Optional[str]
    total_price_usd: Optional[str]
    checkout_token: Optional[str]
    reference: Optional[str]
    user_id: Optional[str]
    location_id: Optional[str]
    source_identifier: Optional[str]
    source_url: Optional[str]
    processed_at: Optional[str]
    device_id: Optional[str]
    phone: Optional[str]
    customer_locale: Optional[str]
    app_id: Optional[str]
    browser_ip: Optional[str]
    landing_site_ref: Optional[str]
    order_number: Optional[str]
    discount_applications: Optional[List[DiscountApplications]]
    discount_codes: Optional[List[DiscountCodes]]
    note_attributes: Optional[List[Properties]]
    payment_gateway_names: Optional[List[Optional[str]]]
    processing_method: Optional[str]
    checkout_id: Optional[str]
    source_name: Optional[str]
    fulfillment_status: Optional[str]
    tax_lines: Optional[List[TaxLines]]
    tags: Optional[str]
    contact_email: Optional[str]
    order_status_url: Optional[str]
    presentment_currency: Optional[str]
    total_line_items_price_set: Optional[AmountSet]
    total_discounts_set: Optional[AmountSet]
    total_shipping_price_set: Optional[AmountSet]
    subtotal_price_set: Optional[AmountSet]
    total_price_set: Optional[AmountSet]
    total_tax_set: Optional[AmountSet]
    total_tip_received: Optional[str]
    line_items: Optional[List[LineItems]]
    shipping_lines: Optional[List[ShippingLine]]
    billing_address: Optional[Address]
    shipping_address: Optional[Address]
    fulfillments: Optional[List[Fulfillments]]
    refunds: Optional[List[Refunds]]
    customer: Optional[CustomerShopify]
    admin_graphql_api_id: Optional[str]




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shopipy',
 'shopipy.resources',
 'shopipy.resources.collections',
 'shopipy.resources.common',
 'shopipy.resources.customers',
 'shopipy.resources.fulfillment',
 'shopipy.resources.inventory',
 'shopipy.resources.oauth',
 'shopipy.resources.orders',
 'shopipy.resources.products',
 'shopipy.resources.shipping',
 'shopipy.resources.transactions',
 'shopipy.resources.webhooks']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.8.2,<2.0.0', 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'shopipy',
    'version': '0.4.0',
    'description': 'An alternative wrapper to the Shopify API.',
    'long_description': None,
    'author': 'Lowercase',
    'author_email': 'lowercase00@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from pydantic import BaseModel
from datetime import datetime
from typing import List, Optional


class HarmonizedCode(BaseModel):
    country_code: str
    harmonized_system_code: str


class InventoryItem(BaseModel):
    cost: str
    country_code_of_origin: str
    country_harmonized_system_codes: Optional[List[Optional[HarmonizedCode]]]
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    harmonized_system_code: Optional[int]
    id: Optional[int]
    province_code_of_origin: Optional[str]
    sku: Optional[str]
    tracked: Optional[bool]
    requires_shipping: Optional[bool]


class InventoryLevel(BaseModel):
    inventory_item_id: Optional[int]
    available: Optional[int]
    location_id: Optional[int]
    updated_at: Optional[datetime]


class InventoryLocation(BaseModel):  
    active: Optional[bool]
    id: Optional[int]
    legacy: Optional[bool]
    updated_at: Optional[datetime]
    created_at: Optional[datetime]
    address1: Optional[str]
    address2: Optional[str]
    city: Optional[str]
    country: Optional[str]
    country_code: Optional[str]
    name: Optional[str]
    phone: Optional[str]
    province: Optional[str]
    province_code: Optional[str]
    zip: Optional[str]
    localized_country_name: Optional[str]
    localized_province_name: Optional[str]


from enum import Enum

ALL_TOPICS = [
    # Admin
    "app/uninstalled",
    "shop/update",
    "bulk_operations/finish",

    # Collections
    "collections/create",
    "collections/delete",
    "collections/update",

    # Customers
    "customers/create",
    "customers/delete",
    "customers/disable",
    "customers/enable",
    "customers/update",

    # Draft orders
    "draft_orders/create",
    "draft_orders/delete",
    "draft_orders/update",

    # Filfillment
    "fulfillment_events/create",
    "fulfillment_events/delete",
    "fulfillments/create",
    "fulfillments/update",

    # Inventory
    "inventory_items/create",
    "inventory_items/delete",
    "inventory_items/update",
    "inventory_levels/connect",
    "inventory_levels/disconnect",
    "inventory_levels/update",

    # Locations
    "locations/create",
    "locations/delete",
    "locations/update",

    # Orders
    "orders/create",
    "orders/cancelled",
    "orders/delete",
    "orders/edited",
    "orders/updated",
    "orders/fulfilled",
    "orders/paid",
    "orders/partially_fulfilled",
    "order_transactions/create",

    # Products
    "products/create",
    "products/delete",
    "products/update"

    # Product Listing
    "product_listings/add",
    "product_listings/remove",
    "product_listings/update",
    
    # Collection Listing
    "collection_listings/add",
    "collection_listings/remove",
    "collection_listings/update",
    
    # Customer Groups
    "customer_groups/create",
    "customer_groups/delete",
    "customer_groups/update",
    "refunds/create",
    "customer_payment_methods/create",
    "customer_payment_methods/revoke",
    "customer_payment_methods/update",
    "carts/create",
    "carts/update",
    "checkouts/create",
    "checkouts/delete",
    "checkouts/update",
    "disputes/create",
    "disputes/update",
    "domains/create",
    "domains/destroy",
    "domains/update",  
    "locales/create",
    "locales/update",    
    "profiles/create",
    "profiles/delete",
    "profiles/update",
    "selling_plan_groups/create",
    "selling_plan_groups/delete",
    "selling_plan_groups/update",
    "customers_marketing_consent/update",
    "subscription_billing_attempts/challenged",
    "subscription_billing_attempts/failure",
    "subscription_billing_attempts/success",
    "subscription_contracts/create",
    "subscription_contracts/update",
    "tender_transactions/create",
    "themes/create",
    "themes/delete",
    "themes/publish",
    "themes/update"
]


from datetime import datetime
from shopipy.base import BaseClient
from .schemas import ShopifyOrders

class Orders(BaseClient):
    orders_path = "/orders"

    def __init__(self, *args, **kwargs):
        super(Orders, self).__init__(*args, **kwargs)

    def count(self, status: str = "any"):
        count_path = self.orders_path + "/count.json"
        return self._get(path=count_path, status=status)

    def find(self, order_id: str):
        find_path = self.orders_path + f"/{order_id}.json"
        return self._get(path=find_path)

    def get(
        self,
        attribution_app_id: str = None,
        created_at_max: datetime = None,
        created_at_min: datetime = None,
        fields: str = None,
        financial_status: str = None,
        fulfillment_status: str = None,
        ids: str = None,
        limit: int = 250,
        processed_at_max: datetime = None,
        processed_at_min: datetime = None,
        since_id: int = None,
        status: str = "any",
        updated_at_max: datetime = None,
        updated_at_min: datetime = None
    ):
        params = dict(
            attribution_app_id=attribution_app_id,
            created_at_max=created_at_max,
            created_at_min=created_at_min,
            fields=fields,
            financial_status=financial_status,
            fulfillment_status=fulfillment_status,
            ids=ids,
            limit=limit,
            processed_at_max=processed_at_max,
            processed_at_min=processed_at_min,
            since_id=since_id,
            status=status,
            updated_at_max=updated_at_max,
            updated_at_min=updated_at_min
        )
        cleaned = self._clean_params(params)
        return self._get(path=f"{self.orders_path}.json", **cleaned)

    def get_all(self, serialize: bool = False):
        all_orders = []
        last_id = 0
        while True:
            try:
                response = self.get(limit=250, since_id=last_id)
            except Exception as error:
                print(str(error))
                continue
            
            if len(response['orders']) == 0:
                break
            
            if serialize:
                schemas = [ShopifyOrders(**single_order) for single_order in response['orders']]
                all_orders.extend(schemas)
                last_id = response['orders'][len(response['orders']) - 1].id # type: ignore
            
            else:
                all_orders.extend(response['orders'])
                last_id = response['orders'][len(response['orders']) - 1]['id']
            
        return all_orders